/**
 * @file
 * JS extensions for quote_plus_js module.
 */

// Create our own namespace and specialized functions.
Drupal.quotePlusJs = {

  /**
   * Make an Ajax request to the module callbacks.
   */
  ajaxRequest: function(caller, appendContent) {
    var cid = Drupal.quotePlusJs.extractId(caller, 'cid');
    var url = Drupal.settings.quotePlusJs.callbackPath + Drupal.quotePlusJs.extractId(caller, 'nid') + '/' + cid;
    $.ajax({
      type: 'POST',
      dataType: 'json',
      url: Drupal.settings.basePath + url,
      success: function(data) {
        Drupal.quotePlusJs.updateForm(cid, data, appendContent);
      }
    });
  },

  /**
   * Extract the nid/cid from a link's CSS class.
   */
  extractId: function(caller, type) {
    var ret = '';
    var regex = new RegExp('\\b(multi)?quote-' + type + '-([0-9]+)');
    var ids = caller.attr('class').match(regex);
    if (ids) {
      ret = ids[2];
    }
    return ret;
  },

  /**
   * Change form data on successful Ajax operation.
   *
   * This callback processes instant reply and instant quote results.
   * It changes the comment form's subject and body to the data
   * retrieved from the server callback.
   *
   * It also changes the form's "referer" field which will help
   * to properly indicate which content the new comment refers to.
   * This is important as the comment form, when residing on the node page
   * below the node and it's comments, by default considers comments always
   * referring to the node itself whyle the user may have clicked "reply to
   * comment".
   */
  updateForm: function(cid, data, appendContent) {
    if (data) {
      if (data.subject != '') {
        $('input#edit-subject').val(data.subject);
      }

      // WYSIWYG support. Inspired by http://drupal.org/project/comment_easy_reply.
      var textareaid = 'edit-comment';
      var focused = false;
      if (Drupal.wysiwyg && Drupal.wysiwyg.activeId && $('textarea#edit-comment').css('display') == 'none') {
        // setContent() method has been added very late to wysiwyg API.
        // To remedy this for older versions, we rather append the content
        // than risking an exception.
        if (appendContent || typeof Drupal.wysiwyg.instances[Drupal.wysiwyg.activeId].setContent != 'function') {
          Drupal.wysiwyg.instances[Drupal.wysiwyg.activeId].insert(data.wysiwyg);
        }
        else {
          Drupal.wysiwyg.instances[Drupal.wysiwyg.activeId].setContent(data.wysiwyg);
        }
      }
      // CKeditor module support.
      else if (typeof(CKEDITOR) != 'undefined' && CKEDITOR.instances[textareaid] && $('textarea#edit-comment').css('display') == 'none') {
        if (appendContent) {
          CKEDITOR.instances[textareaid].insertHtml(data.wysiwyg);
        }
        else {
          CKEDITOR.instances[textareaid].setData(data.wysiwyg);
        }
      }
      // TinyMCE module support.
      else if (typeof(tinyMCE) != 'undefined' && tinyMCE.activeEditor && $('textarea#edit-comment').css('display') == 'none') {
        if (appendContent) {
          tinyMCE.activeEditor.execCommand('mceInsertContent', false, data.wysiwyg);
        }
        else {
          tinyMCE.activeEditor.execCommand('mceSetContent', false, data.wysiwyg);
        }
      }
      // FCKeditor module support.
      else if (typeof(FCKeditorAPI) != 'undefined' && typeof(fckActiveId) != 'undefined' && $('textarea#edit-comment').css('display') == 'none') {
        if (appendContent) {
          FCKeditorAPI.Instances[fckActiveId].InsertHtml(data.wysiwyg);
        }
        else {
          FCKeditorAPI.Instances[fckActiveId].setHTML(data.wysiwyg);
        }
      }
      // No editor found, go for plain textarea.
      else {
        if (appendContent) {
          data.content = $('textarea#edit-comment').val() + '\n' + data.content;
        }
        $('textarea#edit-comment').val(data.content).scrollTop(
          $("textarea#edit-comment").attr("scrollHeight")
        ).focus();
        focused = true;
      }
      // Focus the editor.
      if (!focused) {
        $('div#edit-comment-wrapper').scrollTop($('div#edit-comment-wrapper').attr("scrollHeight"));
      }
      $('input#edit-pid').val(cid);
    }
  }
}

Drupal.behaviors.quotePlusJs = function(context) {

  // Create dynamic links and attach events. This all will only work
  // if we have the comment form residing on the same page, of course.
  if ($('form#comment-form').length > 0) {

    // Instant quote links.
    if (Drupal.settings.quotePlusJs.instantQuotes) {
      $("a.quote-plus-js.quote", context).each(function() {
        $(this).click(function(event) {
          $('p#comment-form-replacement').hide();
          $('form#comment-form').show();
          Drupal.quotePlusJs.ajaxRequest($(this));
          event.preventDefault();
        });
      });
    }

    // Multiquote links. Transform the disabled HTML-spans into working links.
    if (Drupal.settings.quotePlusJs.instantMultiquotes) {
      $("a.quote-plus-js.multiquote", context).each(function() {
        $(this).click(function(event) {
          $('p#comment-form-replacement').hide();
          $('form#comment-form').show();
          Drupal.quotePlusJs.ajaxRequest($(this), true);
          event.preventDefault();
        });
      });
    }

    // Change and hide form if configured.
    if (Drupal.settings.quotePlusJs.alterForm) {
      // Remove a replacement probably made by comment_plus_js.
      $('p#comment-form-replacement').remove();

      // Build new replacement.
      $('form#comment-form', context).hide().before(
        '<p id="comment-form-replacement">' + Drupal.settings.quotePlusJs.formReplacementText + '</p>'
      );
    }
  }
}
